var express = require('express');
var router = express.Router();
var mongodb = require('mongodb');
var mc = mongodb.MongoClient;
var ObjectID = mongodb.ObjectID;

var notesCollection;

var connectToDBs = function(callback) {
    mc.connect('mongodb://localhost/persistent-notes', function(err, db) {
	if (err) {
	    throw err;
	}
	
	notesCollection = db.collection('notes');

	if (callback) {
	    callback();
	}
    });
}

// connect to DB when file is loaded
connectToDBs();

router.get('/', function(req, res) {
    if (req.session.username) {
        res.redirect("/notes");
    } else {
	res.render('index', { title: 'COMP 2406 Persistent Notes Demo', 
			      error: req.query.error });
    }
});

var escapeText = function(html) {
    var result = String(html)
    .replace(/&/g, '&amp;')
    .replace(/</g, '&lt;')
    .replace(/>/g, '&gt;')
    .replace(/"/g, '&quot;');
    if (result === '' + html) return html;
    else return result;
};

var convertNote = function(n, noteIndex, notes) {
    var content = escapeText(n.content);
    var links = content.split('[');
    var i, s, rightOffset, spaceOffset, url, label, remainder;

    for (i=0; i<links.length; i++) {
	s = links[i];
	rightOffset = s.indexOf(']');
	spaceOffset = s.indexOf(' ');
	if (rightOffset != -1) {
	    if (spaceOffset === -1) {
		url = s.slice(0, rightOffset);
		label = url;
	    } else {
		url = s.slice(0,spaceOffset);
		label = s.slice(spaceOffset + 1, rightOffset);
	    }
	    remainder = s.slice(rightOffset + 1, s.length);
	    links[i] = '<a href="' + url + '">' +
		label + '</a>' + remainder;
	}
    }
    
    n.content = links.join('');
    notes[noteIndex] = n;
}

router.get('/notes', function(req, res) {
    var username = req.session.username;

    var renderNotes = function(err, notes) {
	if (err) {
	    notes = [];
	}

	notes.forEach(convertNote);

	res.render("notes.jade", {username: username,
				  title: username +"'s Notes",
				  userNotes: notes,
				  error: req.query.error});
    }
    
    if (username) {
	notesCollection.find({owner: username}).toArray(renderNotes);
    } else {
	res.redirect("/?error=Not Logged In");
    }
});

router.post('/login', function(req, res) {
    var username = req.body.username;
    req.session.username = username;

    res.redirect("/notes")
});

router.post('/logout', function(req, res) {
    req.session.destroy(function(err){
        if (err) {
            console.log("Error: %s", err);
        }
    });
    res.redirect("/");
});

router.get('/edit/:id', function(req, res) {
    var username = req.session.username;
    var id = ObjectID(req.params.id);
    var note;

    var renderEditor = function(err, note) {
	if (err) {
	    res.redirect("/notes?error=Unknown Note to Edit");
	} else {
	    res.render("edit.jade", {title: "Edit Note",
				     note: note});
	}
    }
     
    if (username) {
	notesCollection.findOne({_id: id}, renderEditor)
    } else {
	res.redirect("/?error=Not Logged In");
    }
});

router.get('/delete/:id', function(req, res) {
    var username = req.session.username;
    var id = ObjectID(req.params.id);
    var note;

    var renderDeleteConfirm = function(err, note) {
	if (err) {
	    res.redirect("/notes?error=Unknown Note to Delete");
	} else {
	    res.render("delete.jade", {title: "Delete Note",
				     note: note});
	}
    }
     
    if (username) {
	notesCollection.findOne({_id: id}, renderDeleteConfirm)
    } else {
	res.redirect("/?error=Not Logged In");
    }
});

router.post('/delete', function(req, res) {
    var username = req.session.username;
    var id = ObjectID(req.body.id);
    var noteTitle = req.body.noteTitle;
    
    var resultsPage = function(status) {
	res.render("deleted.jade", {status: status,
				    noteTitle: noteTitle});	
    }

    var checkDelete = function(err, result) {
	if (err) {
	    resultsPage("NOT deleted");
	} else {
	    resultsPage("deleted");
	}
    }
    
    if (username) {
	notesCollection.remove({_id: id},
			       checkDelete);
    } else {
	res.redirect("/?error=Not Logged In");
    }
});

router.post('/update', function(req, res) {
    var username = req.session.username;
    var id = ObjectID(req.body.id);
    var noteTitle = req.body.title;
    var content = req.body.content;
    
    var resultsPage = function(status) {
	res.render("updated.jade", {title: status,
				    noteTitle: noteTitle});	
    }

    var checkUpdate = function(err, result) {
	if (err) {
	    resultsPage("Update Failed");
	} else {
	    resultsPage("Update Succeded");
	}
    }
    
    if (username) {
	notesCollection.update({_id: id},
			       {$set: {title: noteTitle,
				       content: content}},
			       checkUpdate);
    } else {
	res.redirect("/?error=Not Logged In");
    }
});

router.post('/new', function(req, res) {
    var username = req.session.username;
    var nextNote, note;

    var reportInserted = function(err, notesInserted) {
	if (err) {
	    res.redirect("/notes?error=Could not create a new note");
	} else {
	    res.redirect("/edit/" + notesInserted[0]._id);
	}
    }

    if (username) {
	newNote = {title: "Untitled",
		   owner: username,
		   content: "No content"};

	notesCollection.insert(newNote, reportInserted);
    } else {
	res.redirect("/?error=Not Logged In");
    }
});

router.get('/changeusername', function(req, res) {
    var username = req.session.username;

    if (username) {
	res.render("changeusername.jade", {oldusername: username});
    } else {
	res.redirect("/?error=Not Logged In");
    }
});

router.post('/changeusername', function(req, res) {
    var oldUsername = req.session.username;
    var newUsername = req.body.username;

    var reportResult = function(status) {
	res.render("usernamechanged.jade", {status: status,
					    oldusername: oldUsername,
					    newusername: newUsername});
    }

    var usernameUpdated = function(err, result) {
	if (err) {
	    req.session.username = oldUsername;
	    reportResult("FAILED because of database update issue.");
	} else {
	    reportResult("succeeded.");
	}
    }

    var checkNewUsername = function(err, note) {
	if (err) {
	    reportResult("FAILED because database check for new username failed.");
	} else {
	    if (note) {
		reportResult("FAILED because notes are already owned by " +
			     newUsername + ".");
	    } else {
		req.session.username = newUsername;
		notesCollection.update({owner: oldUsername},
				       {$set: {owner: newUsername}},
				       {multi: true},
				       usernameUpdated);
	    }
	}
    }

    if (oldUsername) {
	notesCollection.findOne({owner: newUsername}, checkNewUsername);
    } else {
	res.redirect("/?error=Not Logged In");
    }
});

module.exports = router;
